@@ -2,10 +2,16 @@ package CGI::Session::Test::Default;
 
 use strict;
 use Carp;
-use Test::More;
+use Test::More ();
 use Data::Dumper;
+use Scalar::Util "refaddr";
 
-$CGI::Session::Test::Default::VERSION = '1.52';
+our $AUTOLOAD;
+our $CURRENT;
+sub ok_later (&;$);
+    
+
+$CGI::Session::Test::Default::VERSION = '4.29_1';
 
 =head1 CGI::Session::Test::Default
 
@@ -29,9 +35,16 @@ sub new {
     my $self    = bless {
             dsn     => "driver:file",
             args    => undef,
-            tests   => 99,
+            tests   => 101,
+            test_number =>  0,
             @_
     }, $class;
+    
+    if($self->{skip}) {
+        $self->{_skip} = { map { $_ => $_ } @{$self->{skip}} };
+    } else {
+        $self->{_skip} = {};
+    }
 
     return $self;
 }
@@ -66,6 +79,7 @@ Run the test suite. See C<new()> for setting related options.
 sub run {
     my $self = shift;
 
+    $CURRENT = $self;
     use_ok("CGI::Session", "CGI::Session loaded successfully!");
 
     my $sid = undef;
@@ -73,9 +87,9 @@ sub run {
         ok(1, "=== 1 ===");
         my $session = CGI::Session->load() or die CGI::Session->errstr;
         ok($session, "empty session should be created");
-        ok(!$session->id);
-        ok($session->is_empty);
-        ok(!$session->is_expired);
+        ok(!$session->id, 'Id is empty');
+        ok($session->is_empty, 'Session is empty');
+        ok(!$session->is_expired, 'Session is not expired');
 
         undef $session;
 
@@ -126,14 +140,14 @@ sub run {
             eval { $session = CGI::Session->load($self->{dsn}, $sid, $self->{args}) };
 
             if ($@ || CGI::Session->errstr) {
-                skip "couldn't load session, bailing out: SQLite/Storable support is TODO", 56;
+                Test::More::skip("couldn't load session, bailing out: SQLite/Storable support is TODO", 56);
             }
 
             is($@.CGI::Session->errstr,'','survived eval without error.');
             ok($session, "Session was retrieved successfully");
             ok(!$session->is_expired, "session isn't expired yet");
 
-            is($session->id,$sid, "session IDs are consistent: " . $session->id);
+            is($session->id,$sid, "session IDs are consistent");
             ok($session->atime > $session->ctime, "ctime should be older than atime");
             ok(!$session->etime, "etime shouldn't be set yet");
 
@@ -257,7 +271,7 @@ sub run {
         ok($session->param('simple_object')->name eq "Sherzod Ruzmetov");
         ok($session->param('simple_object')->emails(1) eq 'sherzodr@cpan.org');
         ok($session->param('simple_object')->blogs('yigitlik') eq 'http://author.handalak.com/uz/');
-
+        
         #
         # creating an overloaded object to be stored into session
         my $overloaded_class = OverloadedObjectClass->new("ABCDEFG");
@@ -276,7 +290,8 @@ sub run {
         $simple_class2->emails(1, 'sherzodr@cpan.org');
         $simple_class2->blogs('lost+found', 'http://author.handalak.com/');
         $simple_class2->blogs('yigitlik', 'http://author.handalak.com/uz/');
-        $session->param("embedded_simple_and_overloaded",[ undef, $simple_class2, OverloadedObjectClass->new("Embedded") ]);
+        my $embedded = OverloadedObjectClass->new("Embedded");
+        $session->param("embedded_simple_and_overloaded",[ undef, $simple_class2, $embedded, $embedded ]);
 
         ok(!defined($session->param("embedded_simple_and_overloaded")->[0]),"First element of anonymous array undef");
 
@@ -285,6 +300,9 @@ sub run {
         ok($session->param("embedded_simple_and_overloaded")->[1]->blogs('yigitlik') eq 'http://author.handalak.com/uz/');
   
         ok($session->param("embedded_simple_and_overloaded")->[2] eq "Embedded");
+        
+        ok(refaddr($session->param("embedded_simple_and_overloaded")->[2]) == refaddr($session->param("embedded_simple_and_overloaded")->[3] ),
+            "Overloaded objects have matching addresses");
     }
 
 
@@ -299,10 +317,10 @@ sub run {
         ok(ref $simple_object eq "SimpleObjectClass", "SimpleObjectClass loaded successfully");
 
         my $dsn = CGI::Session->parse_dsn($self->{dsn});
-        ok($simple_object->name eq "Sherzod Ruzmetov");
-        ok($simple_object->emails(1) eq 'sherzodr@cpan.org');
-        ok($simple_object->emails(0) eq 'sherzodr@handalak.com');
-        ok($simple_object->blogs('lost+found') eq 'http://author.handalak.com/');
+        ok_later { $simple_object->name eq "Sherzod Ruzmetov" };
+        ok_later { $simple_object->emails(1) eq 'sherzodr@cpan.org' };
+        ok_later { $simple_object->emails(0) eq 'sherzodr@handalak.com' };
+        ok_later { $simple_object->blogs('lost+found') eq 'http://author.handalak.com/' };
         ok(ref $session->param("overloaded_object") );
         ok($session->param("overloaded_object") eq "ABCDEFG", "Object is still overloaded");
         ok(overload::Overloaded($session->param("overloaded_object")), "Object is really overloaded");
@@ -312,18 +330,61 @@ sub run {
         my $simple_object2 = $session->param("embedded_simple_and_overloaded")->[1];
         ok(ref $simple_object2 eq "SimpleObjectClass", "SimpleObjectClass loaded successfully");
 
-        ok($simple_object2->name eq "Sherzod Ruzmetov");
-        ok($simple_object2->emails(1) eq 'sherzodr@cpan.org');
-        ok($simple_object2->emails(0) eq 'sherzodr@handalak.com');
-        ok($simple_object2->blogs('lost+found') eq 'http://author.handalak.com/');
+        ok_later { $simple_object2->name eq "Sherzod Ruzmetov" };
+        ok_later { $simple_object2->emails(1) eq 'sherzodr@cpan.org' };
+        ok_later { $simple_object2->emails(0) eq 'sherzodr@handalak.com' };
+        ok_later { $simple_object2->blogs('lost+found') eq 'http://author.handalak.com/' };
 
         
         ok($session->param("embedded_simple_and_overloaded")->[2] eq "Embedded");
         ok(overload::Overloaded($session->param("embedded_simple_and_overloaded")->[2]), "Object is really overloaded");
+        
+        ok(refaddr($session->param("embedded_simple_and_overloaded")->[2]) == refaddr($session->param("embedded_simple_and_overloaded")->[3]),
+            "Overloaded objects have matching addresses");        
         $session->delete();
     }
+    
+    $CURRENT = undef;
+    $self->{test_number} = 0;
 }
 
+sub skip_or_run {
+    my $test = shift;
+    
+    $CURRENT->{test_number} ++;
+
+    SKIP: {
+        if($CURRENT->{_skip}->{$CURRENT->{test_number}}) {
+            Test::More::skip("Test does not apply to this setup.", 1);
+        }
+        
+        no strict 'refs';
+        &{"Test::More::$test"}(@_);
+    }
+}
+
+sub ok { skip_or_run("ok", @_); }
+sub use_ok { skip_or_run("use_ok", @_); }
+sub is { skip_or_run("is", @_); }
+
+sub ok_later (&;$) {
+    my($code, $name) = @_;
+    
+    $CURRENT->{test_number} ++;
+    $name = '' unless $name;
+
+    SKIP: {
+        if($CURRENT->{_skip}->{$CURRENT->{test_number}}) {
+            Test::More::skip("Test does not apply to this setup.", 1);
+            fail($name);
+        } else {
+            Test::More::ok($code->(), $name);
+        }
+    }
+}
+
+sub DESTROY { 1; }
+
 
 package SimpleObjectClass;
 use strict;
@@ -345,7 +406,6 @@ use overload (
     'eq'    => \&equals
 );
 
-
 sub new {
     return bless {
         str_value => $_[1]
@@ -363,5 +423,4 @@ sub equals {
     return ($self->as_string eq $arg);
 }
 
-
 1;
